"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const change_case_1 = require("change-case");
const handlebars = __importStar(require("handlebars"));
const prettier = __importStar(require("prettier"));
const loading_1 = require("amplify-graphql-docs-generator/lib/generator/utils/loading");
const generator_1 = __importStar(require("amplify-graphql-docs-generator/lib/generator"));
const constants_1 = require("../constants");
const types_1 = require("../types");
const TEMPLATE_DIR = path.resolve(path.join(__dirname, "../../templates"));
const DEFAULT_MAX_DEPTH = 3;
const isSupportedLanguage = (language) => Object.values(types_1.Language).includes(language);
exports.generate = async (schemaPath, outputPath, options) => {
    const { language } = options;
    if (!isSupportedLanguage(language)) {
        throw new Error(`${language} is not supported, please use one of ${Object.values(types_1.Language)}.`);
    }
    /**
     * The following is copied from
     * amplify-cli/packages/amplify-graphql-docs-generator/src/index.ts
     */
    const schemaData = loading_1.loadSchema(schemaPath);
    const gqlOperations = generator_1.default(schemaData, DEFAULT_MAX_DEPTH, {
        useExternalFragmentForS3Object: false,
    });
    handlebars.registerHelper("camelCase", change_case_1.camelCase);
    handlebars.registerHelper("format", function (fnOptions) {
        const result = fnOptions.fn();
        return format(result, language);
    });
    /**
     * If the outputPath is not "*.ts" - we need separate files
     */
    const separateFiles = !/\.ts$/g.test(outputPath);
    if (separateFiles) {
        /**
         * Since separate files - ensure containing directory exists
         */
        fs.ensureDirSync(outputPath);
        /**
         * For each gqlOperation - generate hooks and write a file
         */
        for (const gqlOperation of Object.values(generator_1.GQLOperationTypeEnum)) {
            const operationsPerType = generateOperations(gqlOperations, gqlOperation);
            generateFile(operationsPerType, outputPath, options.typesImportPath, options.statementsImportPath, language, constants_1.operationsFilenames[gqlOperation] + "." + constants_1.FILE_EXTENSION_MAP[language]);
        }
    }
    else {
        /**
         * Create an accumulator for all the hooks
         * For each gqlOperation - generate hooks and push them to the accumulator
         * Write a file with all the hooks
         */
        const operationsPerType = [];
        for (const gqlOperation of Object.values(generator_1.GQLOperationTypeEnum)) {
            generateOperations(gqlOperations, gqlOperation).forEach((operationsForType) => operationsPerType.push(operationsForType));
        }
        generateFile(operationsPerType, path.dirname(outputPath), options.typesImportPath, options.statementsImportPath, language, path.basename(outputPath));
    }
};
/**
 * Creates a file for the provides operations
 */
const generateFile = (operationsPerType, outputPath, typesImportPath, statementsImportPath, language, filename) => {
    const input = {
        operationsPerType,
        imports: collectImports(language, operationsPerType, path.relative(outputPath, typesImportPath), path.relative(outputPath, statementsImportPath)),
    };
    /**
     * The following is loosely copied from
     * amplify-cli/packages/amplify-graphql-docs-generator/src/index.ts
     */
    const rendered = render(input, language);
    const filepath = path.join(outputPath, filename);
    fs.writeFileSync(filepath, rendered);
};
/**
 * Generates OperationsForType[] for the provided gqlOperation
 */
const generateOperations = (gqlOperations, gqlOperation) => {
    /**
     * Create an accumulator
     */
    const operationsPerType = [];
    /**
     * Each gqlOperation might have multiple apolloOperations to generate
     * ex. query: [useApolloQuery, useApolloLazyQuery]
     */
    const apolloOperations = constants_1.apolloOperationsPerGQLOperation[gqlOperation];
    for (const apolloOperation of apolloOperations) {
        const ops = gqlOperations[constants_1.operationsFilenames[gqlOperation]];
        if (ops.length) {
            const operations = ops.map((operation) => ({
                hookName: "use" + operation.name + apolloOperation,
                operationName: change_case_1.camelCase(operation.name),
                operationType: operation.name + change_case_1.pascalCase(gqlOperation),
                variablesType: operation.args.length > 0
                    ? operation.name + change_case_1.pascalCase(gqlOperation) + "Variables"
                    : undefined,
            }));
            operationsPerType.push({ gqlOperation, apolloOperation, operations });
        }
    }
    return operationsPerType;
};
function format(str, language) {
    /**
     * The following is loosely copied from
     * amplify-cli/packages/amplify-graphql-docs-generator/src/index.ts
     */
    const parserMap = {
        [types_1.Language.JAVASCRIPT]: "babel",
        [types_1.Language.TYPESCRIPT]: "typescript",
    };
    return prettier.format(str, { parser: parserMap[language] });
}
function render(doc, language) {
    /**
     * The following is loosely copied from
     * amplify-cli/packages/amplify-graphql-docs-generator/src/index.ts
     */
    const templateFiles = {
        [types_1.Language.JAVASCRIPT]: "javascript.hbs",
        [types_1.Language.TYPESCRIPT]: "typescript.hbs",
    };
    const templatePath = path.join(TEMPLATE_DIR, templateFiles[language]);
    const templateStr = fs.readFileSync(templatePath, "utf8");
    const template = handlebars.compile(templateStr, {
        noEscape: true,
        preventIndent: true,
    });
    const gql = template(doc);
    return format(gql, language);
}
/**
 * Collects imports for operations to be rendered.
 */
const collectImports = (language, operationsPerType, typesRelativePath, statementsRelativePath) => {
    /**
     * Create an accumulator
     */
    const imports = {};
    /**
     * A helper function to push values to unique keys in the accumulator
     */
    const add = (key, value) => {
        if (!imports[key]) {
            imports[key] = [];
        }
        if (!imports[key].includes(value)) {
            imports[key].push(value);
        }
    };
    /**
     * Loop through all operation types and their operations
     */
    operationsPerType.forEach((operationsForType) => {
        operationsForType.operations.forEach((operation) => {
            /**
             * Add apollo imports
             */
            add("@apollo/client", "use" + operationsForType.apolloOperation);
            if (language === types_1.Language.TYPESCRIPT) {
                add("@apollo/client", operationsForType.apolloOperation + "HookOptions");
            }
            if (language === types_1.Language.TYPESCRIPT) {
                /**
                 * Add operation return type import
                 * and operation variables type import if such exist
                 */
                typesRelativePath = ensureRelativePath(typesRelativePath);
                add(typesRelativePath, operation.operationType);
                if (operation.variablesType) {
                    add(typesRelativePath, operation.variablesType);
                }
            }
            /**
             * Create operation gql import by appending gqlOperation type and add it
             */
            statementsRelativePath = ensureRelativePath(statementsRelativePath);
            let opsFilename = path.join(statementsRelativePath, constants_1.operationsFilenames[operationsForType.gqlOperation]);
            add(opsFilename, operation.operationName);
        });
    });
    return imports;
};
/**
 * Checks if the path starts with a ".", and if not - prepeds "./"
 */
const ensureRelativePath = (pathToCheck) => pathToCheck[0] !== "." ? "." + path.sep + pathToCheck : pathToCheck;
//# sourceMappingURL=index.js.map